require "keyboard"
require "act"
require "timer"
require "player"
require "leaforb"
require "resolution"
require "camera"
require "hitbox"
require "menu"
require "rockodile"
require "lvl"
require "utility"
require "text"
require "trigger"

run = {}

vsyncBool = true
fullscreenBool = true
exactRes = false

function love.run() --removed the 0.001 sleep

    if love.math then
        love.math.setRandomSeed(os.time())
    end

    if love.event then
        love.event.pump()
    end

    if love.load then love.load(arg) end

    -- We don't want the first frame's dt to include time taken by love.load.
    if love.timer then love.timer.step() end

    local dt = 0

    -- Main loop time.
    while true do
        -- Process events.
        if love.event then
            love.event.pump()
            for e,a,b,c,d in love.event.poll() do
                if e == "quit" then
                    if not love.quit or not love.quit() then
                        if love.audio then
                            love.audio.stop()
                        end
                        return
                    end
                end
                love.handlers[e](a,b,c,d)
            end
        end

        -- Update dt, as we'll be passing it to update
        if love.timer then
            love.timer.step()
            dt = love.timer.getDelta()
        end

        -- Call update and draw
        if love.update then love.update(dt) end -- will pass 0 if love.timer is disabled

        if love.window and love.graphics and love.window.isCreated() then
            love.graphics.clear()
            love.graphics.origin()
            if love.draw then love.draw() end
            love.graphics.present()
        end

       --if love.timer then love.timer.sleep(0.001) end
    end
end

function love.load()
    if exactRes == true then
        desktopResX = 1920
    else
        desktopResX = love.window.getDesktopDimensions() * 0.5
    end
    desktopResX , desktopResY = desktopResX , (desktopResX/16*9)
    love.window.setMode(desktopResX,desktopResY,{fullscreen=fullscreenBool,fullscreentype="desktop",vsync=vsyncBool,fsaa=8,resizable=true,minwidth=640,minheight=360,highdpi=true})
    print("Width: "..desktopResX,"Height: "..desktopResY)
    print(vsyncBool)
    --collectgarbage("stop")
    love.mouse.setVisible(hide)
    love.keyboard.setKeyRepeat(true)
    seedSet = true
    gamestate = "playing"
    leaforb.load()
    rockodile.load()
    text.load()
    mainFont = love.graphics.newFont(24)
    love.graphics.setBackgroundColor(255,247,245)
    trigger.load()
    player.load()
    --love.system.openURL("http://google.com/")
end





function love.update(dt)
    
    diffDT = 20
    if dt > 1/(60-diffDT) or dt < 1/(60+diffDT) then
        print("Time "..timer.real.." FPS: "..1/dt)
    end
    if love.keyboard.isDown ("rctrl") then 
    --  debug.debug()
    end
    if seedSet == true then
        seedSet = false
        osTime = os.time()
        randomSetSeed = (dt-0.000005) * osTime
        math.randomseed(randomSetSeed)
        print(randomSetSeed)
    end


    TIMER_UPDATE(dt)
    
    run.update()
    --constantFPS(65,0.5) -- FPS , Sleep % from 0(Diabled, max CPU usage) to 1(CPU Sleep 100% of a frame)
end

function run.load()

end

function run.checkDT()

end

function run.update()
    
    UPDATE_KEYBOARD()
    if gamestate == "playing" then
        PLAYER_UPDATE()
        LEAFORB_UPDATE()
        ROCKODILE_UPDATE()
    	UPDATE_ACT()
        trigger.update()
    end
    if gamestate == "menu" then
        MENU_UPDATE()
    else
        menu.selection = 1
    end


    
    camera_update()



end


function love.draw()

    resolution_set()
    
    --if gamestate == "playing" then
        camera_set()
        
        trigger.draw()
        LVL_DRAW()
        
        LEAFORB_DRAW()
        ROCKODILE_DRAW()
        PLAYER_DRAW()
        text.logic()

        camera_unset()
     
    --end
    
    resolution_unset()
       -- Text debug
    love.graphics.setFont(mainFont)
    love.graphics.setColor(86,86,40)
    --DRAW_KEYBOARD()
    
    
    text.draw()

    --love.graphics.print("Current FPS: "..love.timer.getFPS(),resolution.textHud(5,4))
    --love.graphics.print("Memory MB: "..(math.floor(collectgarbage("count")/(1000/1024)))/1000,resolution.textHud(5,34))
    --love.graphics.print("Timer: "..timer.real,resolution.textHud(5,64))
    --love.graphics.print("PixelScale: "..love.window.getPixelScale(),resolution.textHud(5,94))
    --love.graphics.print(menu.selection,resolution.textHud(5,124))
    --love.graphics.print(keypress,resolution.textHud(5,154))
    
    resolution_set()
    if gamestate == "menu" then
        MENU_DRAW()
    end
    resolution_unset()
end

function constantFPS(fps_constantFPS,tolerance_constantFPS)
    do
        local fps , tolerance , loopTime = fps_constantFPS , tolerance_constantFPS , loopTime_constantFPS
        if loopTime == nil then loopTime = 0 end
        love.timer.sleep((1/fps)*tolerance)
        local loopTime = loopTime - love.timer.getTime()
        local loopDelay = love.timer.getTime() + loopTime
        while love.timer.getTime() - loopDelay < (1/fps) do end
    end
    loopTime_constantFPS = love.timer.getTime()
end